#!/usr/bin/python

from mod_python import Cookie

from session import *
from cache import cache_root

class AuthMyProxy:
	def __init__(self):
		self.host = "imec.grid"

	def auth(self, username, password, session):
		try:
			from myproxy_logon import myproxy_logon
			session.proxy = '%s/%s.pem' % (cache_root, session.session)
			session.files.append(session.proxy)
			myproxy_logon(self.host, username, password, outfile = session.proxy)
			return session
		except Exception, e:
			raise RuntimeError("Unable to authenticate to MyProxy at %s: %s" % (self.host, e))

class AuthRaise:
	def auth(self, username, password, session):
		raise RuntimeException("Default scheme is not implemented")

class SessionGenHMAC:
	def __init__(self):
		import hmac
		self.__hmac = hmac.HMAC("MGP Session Managment")
	def generate(self):
		hmac = self.__hmac.copy()
		import time
		hmac.update(str(time.time()))
		return hmac.hexdigest()

class AuthObj:
	from cache import CachedSession
	cookie_name = "MGP_SESSION_ID"
	__auth = {}
	__auth['default'] = AuthRaise()
	__auth["myproxy"] = AuthMyProxy()

	__session_gen = SessionGenHMAC()
	__session_obj = CachedSession
	session_gen = __session_gen.generate

	def __init__(self, req, safe = False, validate = True):
		self.cookie = Cookie.get_cookies(req)
		self.req = req
		self.session = None
		self.session_name = None
		try:
			if self.cookie.has_key(self.cookie_name):
				self.session_name = self.cookie[self.cookie_name].value
			if self.session_name:
				self.session = self.__session_obj(self.session_name, load = True)
		except Exception, e:
			if not safe:
				raise e
		if validate:
			self.validate()
		
	def auth(self, username, password, scheme = "default"):
		try:
			self.session_fini()
			if not self.__auth.has_key(scheme):
				raise RuntimeError("Auth scheme '%s' is not defined" % scheme)
			session = self.__session_obj(self.session_gen(), load = False)
			session.user = username
			session.scheme = scheme
			session.auth = "auth"
			self.session = self.__auth[scheme].auth(username, password, session)
			self.session_name = session
			self.session_init()
		except Exception, e:
			self.session_fini()
			raise e
	def validate(self):
		try:
			if not self.session:
				raise RuntimeError("Validation failed: User is not logged in")
			self.session.validate()
			self.session.sanitize()
			self.session.store()
			""" validataion code """
			pass
		except Exception, e:
			self.session_fini()
			raise e
	def logout(self):
		self.session_fini()
		if self.session:
			self.session.cleanup()
			self.session = None
	def session_fini(self):
		session = Cookie.Cookie(self.cookie_name, "")
		session.discard = "yes"
		Cookie.add_cookie(self.req, session)
	def session_init(self):
		from time import time
		session = Cookie.Cookie(self.cookie_name, self.session.session)
		session.expires = time() + 3600
		Cookie.add_cookie(self.req, session)
	def get_proxy_file(self):
		if not self.session: return None
		if self.session.proxy: return self.session.proxy
		return None

